@@ -37,8 +37,38 @@
 #include "gdm-net.h"
 #include "gdmconfig.h"
 
-/* Kind of a weird setup, new connections whack old connections */
-#define MAX_CONNECTIONS 10
+/*
+ * Kind of a weird setup, new connections whack old connections.
+ *
+ * We may want to allow tuning of the max connections, since
+ * this number means that only a certain number of slaves can
+ * talk to the daemon at once.  Note that since new connections
+ * whack old connections, this tends to cause traffic problems
+ * if the daemon is really being hammered.
+ *
+ * This is because the slaves retry a failed connection 5 times,
+ * though they are at least smart enough to sleep 1 second
+ * between retries if the connection failed on the connect()
+ * call.  But, this means that throwing off a connection causes
+ * that slave to come back in another second and try to
+ * connect again.  So if the daemon is really being hammered,
+ * this just causes more traffic problems.  It's really faster
+ * to let each connection finish.
+ *
+ * This may cause problems for some setups (perhaps terminal
+ * servers) where lots of connections may hit the server at once
+ * and 15 connections may not be enough (especially since the
+ * console login screen may also be using one of them).  Perhaps
+ * this number should be in configuration file so it can be
+ * tuned by the end user?
+ *
+ * If, when you turn on debug, you notice messages like this
+ * in the log, "Closing connection, x subconnections reached"
+ * and some slaves are not working properly, then bumping this
+ * number up is probably a reasonable fix if you can't simply
+ * reduce the socket load the daemon must handle.
+ */
+#define MAX_CONNECTIONS 15
 
 struct _GdmConnection {
 	int fd;
@@ -74,6 +104,21 @@ struct _GdmConnection {
 	GdmDisplay *disp;
 };
 
+int 
+gdm_connection_is_server_busy (GdmConnection *conn) {
+	int max_connections = MAX_CONNECTIONS;
+
+	if (conn->n_subconnections >= (max_connections / 2)) {
+		gdm_debug ("Connections is %d, max is %d, busy TRUE",
+			conn->n_subconnections, max_connections);
+		return TRUE;
+	} else {
+		gdm_debug ("Connections is %d, max is %d, busy FALSE",
+			conn->n_subconnections, max_connections);
+		return FALSE;
+	}
+}
+
 static gboolean
 close_if_needed (GdmConnection *conn, GIOCondition cond, gboolean error)
 {
@@ -83,7 +128,12 @@ close_if_needed (GdmConnection *conn, GIOCondition cond, gboolean error)
 
 	if (cond & G_IO_ERR ||
 	    cond & G_IO_HUP || error) {
-		gdm_debug ("close_if_needed: Got HUP/ERR on %d", conn->fd);
+	        if (cond & G_IO_ERR)
+			gdm_debug ("close_if_needed: Got G_IO_ERR on %d", conn->fd);
+	        if (cond & G_IO_HUP)
+			gdm_debug ("close_if_needed: Got G_IO_HUP on %d", conn->fd);
+		if (error)
+			gdm_debug ("close_if_needed: Got error on %d", conn->fd);
 		conn->source = 0;
 		gdm_connection_close (conn);
 		return FALSE;
@@ -101,13 +151,12 @@ gdm_connection_handler (GIOChannel *source,
 	char *p;
 	size_t len;
 
-	if ( ! (cond & G_IO_IN)) 
+	if ( ! (cond & G_IO_IN))
 		return close_if_needed (conn, cond, FALSE);
 
 	VE_IGNORE_EINTR (len = read (conn->fd, buf, sizeof (buf) -1));
-	if (len <= 0) {
+	if (len <= 0)
 		return close_if_needed (conn, cond, TRUE);
-	}
 
 	buf[len] = '\0';
 
@@ -124,7 +173,7 @@ gdm_connection_handler (GIOChannel *source,
 		    /* cut lines short at 4096 to prevent DoS attacks */
 		    conn->buffer->len > 4096) {
 			conn->close_level = 1;
-			conn->message_count ++;
+			conn->message_count++;
 			conn->handler (conn, conn->buffer->str,
 				       conn->data);
 			if (conn->close_level == 2) {
@@ -196,14 +245,15 @@ gdm_socket_handler (GIOChannel *source,
 		    GIOCondition cond,
 		    gpointer data)
 {
-	int fd;
 	GIOChannel *unixchan;
 	GdmConnection *conn = data;
 	GdmConnection *newconn;
 	struct sockaddr_un addr;
 	socklen_t addr_size = sizeof (addr);
+	int fd;
+	int max_connections;
 
-	if ( ! (cond & G_IO_IN)) 
+	if ( ! (cond & G_IO_IN))
 		return TRUE;
 
 	VE_IGNORE_EINTR (fd = accept (conn->fd,
@@ -235,9 +285,21 @@ gdm_socket_handler (GIOChannel *source,
 					   parent connection */
 
 	conn->subconnections = g_list_append (conn->subconnections, newconn);
-	conn->n_subconnections ++;
-	if (conn->n_subconnections > MAX_CONNECTIONS) {
-		GdmConnection *old = conn->subconnections->data;
+	conn->n_subconnections++;
+
+	/*
+	 * When dynamix servers is turned on, the daemon can be flooded with
+	 * requests and closing a subconnection will typically make the client
+	 * just try and connect again, and worsen the flooding problem.  When
+	 * using dynamic servers, allow more clients to connect at once.  
+	 */
+	max_connections = MAX_CONNECTIONS;
+             
+	if (conn->n_subconnections > max_connections) {
+		GdmConnection *old;
+		gdm_debug ("Closing connection, %d subconnections reached",
+			max_connections);
+		old = conn->subconnections->data;
 		conn->subconnections =
 			g_list_remove (conn->subconnections, old);
 		gdm_connection_close (old);
@@ -274,7 +336,7 @@ gdm_connection_open_unix (const char *sockname, mode_t mode)
 
 try_again:
 	/* this is all for creating sockets in /tmp/ safely */
-	VE_IGNORE_EINTR (unlink (sockname));
+	VE_IGNORE_EINTR (g_unlink (sockname));
 	if G_UNLIKELY (errno == EISDIR ||
 		       errno == EPERM) {
 		/* likely a directory, someone's playing tricks on us */
@@ -284,8 +346,8 @@ try_again:
 			newname = g_strdup_printf ("%s-renamed-%u",
 						   sockname,
 						   (guint)g_random_int ());
-		} while (access (newname, F_OK) == 0);
-		VE_IGNORE_EINTR (rename (sockname, newname));
+		} while (g_access (newname, F_OK) == 0);
+		VE_IGNORE_EINTR (g_rename (sockname, newname));
 		if G_UNLIKELY (errno != 0)
 			try_again_attempts = 0;
 		g_free (newname);
@@ -308,7 +370,7 @@ try_again:
 		return NULL;
 	}
 
-	VE_IGNORE_EINTR (chmod (sockname, mode));
+	VE_IGNORE_EINTR (g_chmod (sockname, mode));
 
 	conn = g_new0 (GdmConnection, 1);
 	conn->disp = NULL;
@@ -381,7 +443,7 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 	GdmConnection *conn;
 	int fd;
 
-	VE_IGNORE_EINTR (unlink (fifo));
+	VE_IGNORE_EINTR (g_unlink (fifo));
 
 	if G_UNLIKELY (mkfifo (fifo, 0660) < 0) {
 		gdm_error (_("%s: Could not make FIFO"),
@@ -397,7 +459,7 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 		return NULL;
 	}
 
-	VE_IGNORE_EINTR (chmod (fifo, mode));
+	VE_IGNORE_EINTR (g_chmod (fifo, mode));
 
 	conn = g_new0 (GdmConnection, 1);
 	conn->disp = NULL;
