@@ -115,6 +115,7 @@ static gchar *GdmFailsafeXserver = NULL;
 static gchar *GdmXKeepsCrashing = NULL;
 static gchar *GdmHalt = NULL;
 static gchar *GdmReboot = NULL;
+static GdmCustomCmd *GdmCustomCommands = NULL;
 static gchar *GdmSuspend = NULL;
 static gchar *GdmServAuthDir = NULL;
 static gchar *GdmMulticastAddr;
@@ -184,6 +185,7 @@ static gboolean GdmAllowRoot = FALSE;
 static gboolean GdmAllowRemoteRoot = FALSE;
 static gboolean GdmAllowRemoteAutoLogin = FALSE;
 static gboolean GdmCheckDirOwner = TRUE;
+static gboolean GdmSupportAutomount = FALSE;
 static gboolean GdmTimedLoginEnable = FALSE;
 static gboolean GdmDynamicXservers = FALSE;
 static gboolean GdmVTAllocation = TRUE;
@@ -343,6 +345,8 @@ is_key (const gchar *key1, const gchar *key2)
 static void 
 gdm_config_init (void)
 {
+   gint i;
+
    type_hash    = g_hash_table_new (g_str_hash, g_str_equal);
    val_hash     = g_hash_table_new (g_str_hash, g_str_equal);
    realkey_hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -374,6 +378,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_CONSOLE_NOTIFY, &GdmConsoleNotify, &bool_type);
    gdm_config_add_hash (GDM_KEY_TIMED_LOGIN_ENABLE, &GdmTimedLoginEnable, &bool_type);
    gdm_config_add_hash (GDM_KEY_CHECK_DIR_OWNER, &GdmCheckDirOwner, &bool_type);
+   gdm_config_add_hash (GDM_KEY_SUPPORT_AUTOMOUNT, &GdmSupportAutomount, &bool_type);
    gdm_config_add_hash (GDM_KEY_XDMCP, &GdmXdmcp, &bool_type);
    gdm_config_add_hash (GDM_KEY_INDIRECT, &GdmIndirect, &bool_type);
    gdm_config_add_hash (GDM_KEY_XDMCP_PROXY, &GdmXdmcpProxy, &bool_type);
@@ -504,6 +509,40 @@ gdm_config_init (void)
       &GdmPreFetchProgram, &string_type);
    gdm_config_add_hash (GDM_KEY_PAM_STACK, &GdmPamStack, &string_type);
 
+   /* custom command string values */
+   GdmCustomCommands = g_new0 (GdmCustomCmd, GDM_CUSTOM_COMMAND_MAX);   
+   gchar *key_string = NULL; 
+   for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+	   /* For each possible custom command */      
+	   key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i); 
+	   GdmCustomCommands[i].command = NULL;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command, &string_type);
+	   
+	   key_string = g_strdup_printf (_("%s%d=Custom_%d"), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i, i); 
+	   GdmCustomCommands[i].command_label = NULL;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command_label, &string_type);
+	   
+	   key_string = g_strdup_printf (_("%s%d=Execute custom command _%d"), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i, i);
+	   GdmCustomCommands[i].command_lr_label = NULL;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command_lr_label, &string_type);
+	   
+	   key_string = g_strdup_printf (_("%s%d=Are you sure?"), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
+	   GdmCustomCommands[i].command_text = NULL;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command_text, &string_type);
+	   
+	   key_string = g_strdup_printf (_("%s%d=Execute custom command %d"), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i, i);
+	   GdmCustomCommands[i].command_tooltip = NULL;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command_tooltip, &string_type);
+	   
+	   key_string = g_strdup_printf (_("%s%d=false"), GDM_KEY_CUSTOM_CMD_NO_RESTART_TEMPLATE, i);
+	   GdmCustomCommands[i].command_no_restart = FALSE;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command_no_restart, &bool_type);  
+
+	   key_string = g_strdup_printf(_("%s%d=false"), GDM_KEY_CUSTOM_CMD_IS_PERSISTENT_TEMPLATE, i);
+	   GdmCustomCommands[i].command_is_persistent = FALSE;
+	   gdm_config_add_hash (key_string, &GdmCustomCommands[i].command_is_persistent, &bool_type); 
+   }   
+
    /* int values */
    gdm_config_add_hash (GDM_KEY_XINERAMA_SCREEN, &GdmXineramaScreen, &int_type);
    gdm_config_add_hash (GDM_KEY_RETRY_DELAY, &GdmRetryDelay, &int_type);
@@ -616,7 +655,7 @@ gdm_get_custom_config (struct stat *statbuf)
  * appended, and never gdm.conf.
  */
 static gchar *
-gdm_get_per_display_custom_config_file (const gchar *display)
+gdm_get_per_display_custom_config_file (gchar *display)
 {
   return g_strdup_printf ("%s%s", custom_config_file, display);
 }
@@ -779,7 +818,7 @@ gboolean gdm_get_value_bool_per_display (gchar *display, gchar *key)
  * value if none exists.  Note that this value needs to be freed,
  * unlike the non-per-display version.
  */
-gchar * gdm_get_value_string_per_display (const gchar *display, gchar *key)
+gchar * gdm_get_value_string_per_display (gchar *display, gchar *key)
 {
    gchar *perdispval;
 
@@ -805,7 +844,7 @@ gchar * gdm_get_value_string_per_display (const gchar *display, gchar *key)
  * sure they are added to the if-test below.
  */
 void
-gdm_config_key_to_string_per_display (const gchar *display, gchar *key, gchar **retval)
+gdm_config_key_to_string_per_display (gchar *display, gchar *key, gchar **retval)
 {
    gchar *file;
    gchar **splitstr = g_strsplit (key, "/", 2);
@@ -816,10 +855,9 @@ gdm_config_key_to_string_per_display (const gchar *display, gchar *key, gchar **
 
    file = gdm_get_per_display_custom_config_file (display);
 
-   if (splitstr != NULL &&
-       (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
-        strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
-        is_key (key, GDM_KEY_PAM_STACK))) {
+   if (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
+       strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
+       is_key (key, GDM_KEY_PAM_STACK)) {
       gdm_config_key_to_string (file, key, retval);
    }
 
@@ -845,7 +883,7 @@ gdm_config_key_to_string (gchar *file, gchar *key, gchar **retval)
    *retval = NULL;
 
    /* Should not fail, all keys should have a category. */
-   if (splitstr == NULL || splitstr[0] == NULL)
+   if (splitstr[0] == NULL)
       return;
 
    /* If file doesn't exist, then just return */
@@ -1026,6 +1064,7 @@ _gdm_set_value_string (gchar *key, gchar *value_in, gboolean doing_update)
    gchar *setting_copy = NULL;
    gchar *temp_string;
    gchar *value;
+   gint i;
 
    if (! ve_string_empty (value_in))
       value = value_in;
@@ -1158,6 +1197,21 @@ _gdm_set_value_string (gchar *key, gchar *value_in, gboolean doing_update)
 
    /* All others */
    } else {
+       for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {	   
+	       gchar * key_string = NULL;
+	       /* For each possible custom command */     
+	       key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+	       if(is_key (key, key_string)) {	       
+		       if (value != NULL)
+			       *setting = ve_get_first_working_command (value, FALSE);
+		       else
+			       *setting = NULL;
+		       
+		       break;
+	       }
+	       g_free(key_string);
+       }
+
        if (value != NULL)
           *setting = g_strdup (value);
        else {
@@ -1191,6 +1245,19 @@ _gdm_set_value_string (gchar *key, gchar *value_in, gboolean doing_update)
          notify_displays_string (GDM_NOTIFY_GTK_MODULES_LIST, *setting);
       else if (is_key (key, GDM_KEY_TIMED_LOGIN))
          notify_displays_string (GDM_NOTIFY_TIMED_LOGIN, *setting);
+
+      for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {	   	   
+	      gchar * key_string = NULL;
+	      /* For each possible custom command */     
+	      key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i); 
+	      if (is_key (key, key_string)) {
+		      g_free(key_string);
+		      key_string = g_strdup_printf (_("%s%d"), GDM_NOTIFY_CUSTOM_CMD_TEMPLATE, i);
+		      notify_displays_string (key_string, *setting);
+		      break;
+	      }
+	      g_free(key_string);
+      }       
    }
 
    if (setting_copy != NULL)
@@ -1706,7 +1773,7 @@ gdm_update_config (gchar* key)
    if (custom_cfg != NULL) {
        gchar **splitstr = g_strsplit (key, "/", 2);
 
-       if (splitstr != NULL && splitstr[0] != NULL) {
+       if (splitstr[0] != NULL) {
           GList *list = ve_config_get_keys (custom_cfg, splitstr[0]);
 
           while (list != NULL) {
@@ -1894,7 +1961,7 @@ gdm_load_config_option (gpointer key_in, gpointer value_in, gpointer data)
       /* First check the custom file */
       if (cfgfiles->custom_cfg != NULL) {
           gchar **splitstr = g_strsplit (key_in, "/", 2);
-          if (splitstr != NULL && splitstr[0] != NULL) {
+          if (splitstr[0] != NULL) {
              GList *list = ve_config_get_keys (cfgfiles->custom_cfg, splitstr[0]);
 
              while (list != NULL) {
