@@ -70,6 +70,8 @@ gboolean GDM_IS_LOCAL          = FALSE;
 static gboolean ignore_buttons = FALSE;
 gboolean GdmHaltFound          = FALSE;
 gboolean GdmRebootFound        = FALSE;
+gboolean *GdmCustomCmdsFound   = NULL;
+gboolean GdmAnyCustomCmdsFound = FALSE;
 gboolean GdmSuspendFound       = FALSE;
 gboolean GdmConfiguratorFound  = FALSE;
 
@@ -79,6 +81,8 @@ GreeterItemInfo *welcome_string_info = NULL;
 extern gboolean session_dir_whacked_out;
 extern gboolean require_quarter;
 extern gint gdm_timed_delay;
+extern GtkButton *gtk_ok_button;
+extern GtkButton *gtk_start_again_button;
 
 gboolean greeter_probably_login_prompt = FALSE;
 
@@ -190,9 +194,16 @@ process_operation (guchar       op_code,
 					gdm_config_get_string (GDM_KEY_SOUND_ON_LOGIN_FILE),
 					gdm_config_get_bool (GDM_KEY_SOUND_ON_LOGIN));
 		greeter_probably_login_prompt = TRUE;
+		if (gtk_start_again_button != NULL)
+	                gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
 	} else {
 		greeter_probably_login_prompt = FALSE;
+		if (gtk_start_again_button != NULL)
+                	gtk_widget_set_sensitive (gtk_start_again_button, TRUE);
 	}
+	if (gtk_ok_button != NULL)
+                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+
 	greeter_ignore_buttons (FALSE);
 
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, TRUE);
@@ -207,8 +218,16 @@ process_operation (guchar       op_code,
     case GDM_NOECHO:
 	tmp = ve_locale_to_utf8 (args);
 
-	if (tmp != NULL && strcmp (tmp, _("Password:")) == 0)
+	if (tmp != NULL && strcmp (tmp, _("Password:")) == 0) {
+		if (gtk_start_again_button != NULL)
+                	gtk_widget_set_sensitive (gtk_start_again_button, TRUE);
 		greeter_probably_login_prompt = FALSE;
+	} else {
+		if (gtk_start_again_button != NULL)
+                	gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
+	}
+	if (gtk_ok_button != NULL)
+                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
 
 	greeter_ignore_buttons (FALSE);
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, FALSE);
@@ -306,6 +325,11 @@ process_operation (guchar       op_code,
 
     case GDM_RESETOK:
 
+	if (gtk_ok_button != NULL)
+                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+	if (gtk_start_again_button != NULL)
+                gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
+
 	conversation_info = greeter_lookup_id ("pam-conversation");
 	
 	if (conversation_info)
@@ -717,6 +741,8 @@ gdm_set_welcomemsg (void)
 static void
 gdm_read_config (void)
 {
+	gint i;
+
 	/* Read config data in bulk */
 	gdmcomm_comm_bulk_start ();
 
@@ -775,6 +801,27 @@ gdm_read_config (void)
 	gdm_config_get_bool   (GDM_KEY_DEFAULT_REMOTE_WELCOME);
 	gdm_config_get_bool   (GDM_KEY_ADD_GTK_MODULES);
 
+	/* Keys for custom commands */
+	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+		gchar * key_string = NULL;
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+        
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+
+		g_free (key_string);
+	}     
+
 	/* Keys not to include in reread_config */
 	gdm_config_get_string (GDM_KEY_SESSION_DESKTOP_DIR);
 	gdm_config_get_string (GDM_KEY_PID_FILE);
@@ -786,6 +833,9 @@ gdm_read_config (void)
 static gboolean
 greeter_reread_config (int sig, gpointer data)
 {
+	gint i;
+	gboolean custom_changed = FALSE;
+
 	/* Read config data in bulk */
 	gdmcomm_comm_bulk_start ();
 
@@ -845,6 +895,41 @@ greeter_reread_config (int sig, gpointer data)
 		_exit (DISPLAY_RESTARTGREETER);
 	}
 
+	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+		gchar *key_string = NULL;
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+
+		g_free (key_string);
+	}     	
+	
+	if(custom_changed){
+		/* Set busy cursor */
+		gdm_common_setup_cursor (GDK_WATCH);
+		
+		gdm_wm_save_wm_order ();
+		gdmcomm_comm_bulk_stop ();
+		
+		_exit (DISPLAY_RESTARTGREETER);
+	}
+
 	gdm_config_reload_string (GDM_KEY_SOUND_PROGRAM);
 	gdm_config_reload_bool   (GDM_KEY_SOUND_ON_LOGIN);
 	gdm_config_reload_string (GDM_KEY_SOUND_ON_LOGIN_FILE);
@@ -1027,6 +1112,7 @@ main (int argc, char *argv[])
   const char *gdm_gtk_theme;
   guint sid;
   int r;
+  gint i;
 
   if (g_getenv ("DOING_GDM_DEVELOPMENT") != NULL)
     DOING_GDM_DEVELOPMENT = TRUE;
@@ -1051,9 +1137,9 @@ main (int argc, char *argv[])
    */
   gdk_init (&argc, &argv);
   if (! DOING_GDM_DEVELOPMENT) {
-     gdm_common_atspi_launch ();
+       gdm_common_atspi_launch ();
   }
-
+  
   gtk_init (&argc, &argv);
 
   gdm_common_setup_cursor (GDK_WATCH);
@@ -1088,6 +1174,7 @@ main (int argc, char *argv[])
   }
   gdm_common_setup_background_color (bg_color);
   greeter_session_init ();
+  greeter_language_initialize_model ();
 
   ve_signal_add (SIGHUP, greeter_reread_config, NULL);
 
@@ -1157,13 +1244,24 @@ main (int argc, char *argv[])
   * is displayed it doesn't show as 0.  Also determine if the Halt,
   * Reboot, Suspend and Configurator commands work.
   */
-  gdm_timed_delay      = gdm_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
-  GdmHaltFound         = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_HALT));
-  GdmRebootFound       = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_REBOOT));
-  GdmSuspendFound      = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_SUSPEND));
-  GdmConfiguratorFound = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_CONFIGURATOR));
+  gdm_timed_delay         = gdm_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+  GdmHaltFound            = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_HALT));
+  GdmRebootFound          = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_REBOOT));
+  GdmSuspendFound         = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_SUSPEND));
+  GdmConfiguratorFound    = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_CONFIGURATOR));
 
+  GdmCustomCmdsFound = g_new0 (gboolean, GDM_CUSTOM_COMMAND_MAX);
+  for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+	  gchar * key_string = NULL;
+	  /*  For each possible custom command */      
+	  key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+	  GdmCustomCmdsFound[i] = gdm_working_command_exists (gdm_config_get_string (key_string));
+	  if (GdmCustomCmdsFound[i])
+		  GdmAnyCustomCmdsFound = TRUE;
 
+	  g_free (key_string);
+  }
+  
   if (g_getenv ("GDM_THEME") != NULL)
      gdm_graphical_theme = g_strdup (g_getenv ("GDM_THEME"));
   else if (gdm_config_get_bool (GDM_KEY_GRAPHICAL_THEME_RAND))
@@ -1343,6 +1441,7 @@ main (int argc, char *argv[])
 
   greeter_item_ulist_unset_selected_user ();
   greeter_item_ulist_enable ();
+  greeter_item_ulist_check_show_userlist ();
 
   /* can it ever happen that it'd be NULL here ??? */
   if G_UNLIKELY (window->window != NULL)
