@@ -36,11 +36,13 @@
 #include "gdmwm.h"
 #include "misc.h"
 
-GtkWidget *dialog;
-extern gboolean GdmHaltFound;
-extern gboolean GdmRebootFound;
-extern gboolean GdmSuspendFound;
-extern gboolean GdmConfiguratorFound;
+GtkWidget       *dialog;
+extern gboolean  GdmHaltFound;
+extern gboolean  GdmRebootFound;
+extern gboolean *GdmCustomCmdsFound;
+extern gboolean  GdmAnyCustomCmdsFound;
+extern gboolean  GdmSuspendFound;
+extern gboolean  GdmConfiguratorFound;
 
 /* doesn't check for executability, just for existance */
 static gboolean
@@ -70,46 +72,49 @@ bin_exists (const char *command)
 static void
 query_greeter_restart_handler (void)
 {
-       GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
-       GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
-
 	if (gdm_wm_warn_dialog (_("Are you sure you want to restart the computer?"), "",
 			     _("_Restart"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		closelog ();
 		
 		_exit (DISPLAY_REBOOT);
 	}
-	gtk_widget_grab_focus (entry);
+}
+
+static void
+query_greeter_custom_cmd_handler (GtkWidget *widget, gpointer data)
+{
+        if (data) {
+		gint *cmd_id = (gint*)data;
+	        gchar * key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, *cmd_id);
+		if (gdm_wm_warn_dialog (gdm_config_get_string (key_string) , "", 
+					GTK_STOCK_OK, NULL, TRUE) == GTK_RESPONSE_YES) {
+		        printf ("%c%c%c%d\n", STX, BEL, GDM_INTERRUPT_CUSTOM_CMD, *cmd_id);
+			fflush (stdout);
+		}
+		g_free (key_string);		
+	}
 }
 
 static void
 query_greeter_halt_handler (void)
 {
-	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
-	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
-
 	if (gdm_wm_warn_dialog (_("Are you sure you want to Shut Down the computer?"), "",
 			     _("Shut _Down"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		closelog ();
 
 		_exit (DISPLAY_HALT);
 	}
-	gtk_widget_grab_focus (entry);
 }
 
 static void
 query_greeter_suspend_handler (void)
 {
-	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
-	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
-
 	if (gdm_wm_warn_dialog (_("Are you sure you want to suspend the computer?"), "",
 			     _("_Suspend"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		/* suspend interruption */
 		printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_SUSPEND);
 		fflush (stdout);
 	}
-	gtk_widget_grab_focus (entry);
 }
 
 static void
@@ -119,6 +124,12 @@ greeter_restart_handler (void)
 	_exit (DISPLAY_REBOOT);
 }
 
+static void
+greeter_custom_cmd_handler (gint cmd_id)
+{
+	printf ("%c%c%c%d\n", STX, BEL, GDM_INTERRUPT_CUSTOM_CMD, cmd_id);
+	fflush (stdout);
+}
 
 static void
 greeter_halt_handler (void)
@@ -161,6 +172,7 @@ void
 greeter_system_append_system_menu (GtkWidget *menu)
 {
 	GtkWidget *w, *sep;
+	gint i = 0;
 
 	/* should never be allowed by the UI */
 	if ( ! gdm_config_get_bool (GDM_KEY_SYSTEM_MENU) ||
@@ -186,7 +198,7 @@ greeter_system_append_system_menu (GtkWidget *menu)
 				  NULL);
 	}
 
-	if (GdmRebootFound || GdmHaltFound || GdmSuspendFound) {
+	if (GdmRebootFound || GdmHaltFound || GdmSuspendFound || GdmAnyCustomCmdsFound) {
 		sep = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), sep);
 		gtk_widget_show (sep);
@@ -201,6 +213,24 @@ greeter_system_append_system_menu (GtkWidget *menu)
 				  NULL);
 	}
 
+	if (GdmAnyCustomCmdsFound) {
+		for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+		        if (GdmCustomCmdsFound[i]){
+				gint * cmd_index = g_new0(gint, 1);
+				*cmd_index = i;
+				gchar * key_string = NULL;
+				key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
+				w = gtk_menu_item_new_with_mnemonic (gdm_config_get_string(key_string));
+				gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
+				gtk_widget_show (GTK_WIDGET (w));
+				g_signal_connect (G_OBJECT (w), "activate",
+						  G_CALLBACK (query_greeter_custom_cmd_handler),
+						  cmd_index);
+				g_free (key_string);
+			}
+		}
+	}
+
 	if (GdmHaltFound) {
 		w = gtk_menu_item_new_with_mnemonic (_("Shut _Down"));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
@@ -244,10 +274,12 @@ greeter_system_handler (GreeterItemInfo *info,
   GtkWidget *halt_radio = NULL;
   GtkWidget *suspend_radio = NULL;
   GtkWidget *restart_radio = NULL;
+  GtkWidget **custom_cmds_radio = NULL;
   GtkWidget *config_radio = NULL;
   GtkWidget *chooser_radio = NULL;
   gchar *s;
   int ret;
+  gint i;
   GSList *radio_group = NULL;
   static GtkTooltips *tooltips = NULL;
 
@@ -328,6 +360,33 @@ greeter_system_handler (GreeterItemInfo *info,
 	  gtk_widget_show (restart_radio);
   }
 
+  if (GdmAnyCustomCmdsFound) {
+	  custom_cmds_radio = g_new0 (GtkWidget*, GDM_CUSTOM_COMMAND_MAX); 
+	  for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+	          custom_cmds_radio[i] = NULL;
+		  if (GdmCustomCmdsFound[i]){
+		          gchar * key_string = NULL;
+			  key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
+			  radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (group_radio));
+			  custom_cmds_radio[i] = gtk_radio_button_new_with_mnemonic (radio_group,
+										     gdm_config_get_string(key_string));
+			  group_radio = custom_cmds_radio[i];
+			  g_free (key_string);
+			  key_string = g_strdup_printf (_("%s%d="), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
+			  gtk_tooltips_set_tip (tooltips, GTK_WIDGET (custom_cmds_radio[i]),
+						gdm_config_get_string(key_string),
+						NULL);
+			  g_signal_connect (G_OBJECT(custom_cmds_radio[i]), "button_press_event",
+					    G_CALLBACK(radio_button_press_event), NULL);
+			  gtk_box_pack_start (GTK_BOX (vbox),
+					      custom_cmds_radio[i],
+					      FALSE, FALSE, 4);
+			  gtk_widget_show (custom_cmds_radio[i]);
+			  g_free (key_string);
+		  }
+	  }
+  }
+
   if (GdmSuspendFound) {
 	  if (group_radio != NULL)
 		  radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (group_radio));
@@ -417,6 +476,11 @@ greeter_system_handler (GreeterItemInfo *info,
     greeter_config_handler ();
   else if (chooser_radio != NULL && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chooser_radio)))
     greeter_chooser_handler ();
+  else
+    for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+	if (custom_cmds_radio[i] != NULL &&  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (custom_cmds_radio[i])))
+	    greeter_custom_cmd_handler (i);
+    }  
 
   gtk_widget_destroy (dialog);
 }
@@ -424,10 +488,12 @@ greeter_system_handler (GreeterItemInfo *info,
 
 void
 greeter_item_system_setup (void)
-{
+{  
+  gint i;
+	
   greeter_item_register_action_callback ("reboot_button",
 					 (ActionFunc)query_greeter_restart_handler,
-					 NULL);
+					 NULL);  
   greeter_item_register_action_callback ("halt_button",
 					 (ActionFunc)query_greeter_halt_handler,
 					 NULL);
@@ -443,4 +509,14 @@ greeter_item_system_setup (void)
   greeter_item_register_action_callback ("chooser_button",
 					 (ActionFunc)greeter_chooser_handler,
 					 NULL);
+
+  for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+	  gint * cmd_index = g_new0(gint, 1);
+	  *cmd_index = i;
+	  gchar * key_string = g_strdup_printf (_("custom_cmd_button%d"), i);
+	  greeter_item_register_action_callback (key_string,
+						 (ActionFunc)query_greeter_custom_cmd_handler,
+						 cmd_index);
+	  g_free (key_string);
+  }
 }
